local DQuest = Dominos:NewModule('Quest')
local Quest = Dominos:CreateClass('Frame', Dominos.Frame)

function DQuest:Load()
	self.frame = Quest:New()
	self.frame:SetFrameStrata('LOW')
end

function DQuest:Unload()
	self.frame:Free()
end

function Quest:New()
	local f = self.super.New(self, 'quest')
	f.Child, f.scrollbar, f.contain = Quest:NewScrollFrame(f)
	f:TrackerSetup()
	f:Layout()

	return f
end

function Quest:GetDefaults()
	return {
		scale = 1,
		point = 'RIGHT',
		x = 0,
		y= 0,
		width = 200,
		height = 394,
		padding = 0,
	}
end

local Clicked

function Quest:Layout()
	local Loffset, Roffset
	self.scrollbar:Show()
	self.scrollbar:ClearAllPoints()
	self.collapse:ClearAllPoints()
	if self.sets.disable then
		Loffset, Roffset = 0,  0
		self.scrollbar:Hide()
		self.scrollbar:SetValue(1)
 		if self.sets.OnLeft then
			self.collapse:SetPoint("BottomLeft", self.Child, "TopLeft")
 		else
			self.collapse:SetPoint("BottomRight", self.Child, "TopRight")
		end		
 	elseif self.sets.OnLeft then
		Loffset, Roffset = 16,  0
		self.scrollbar:SetPoint("TopRight", self.Child, "TopLeft", 0, 3)
		self.scrollbar:SetPoint("BottomRight", self.Child, "BottomLeft", 0, -3)
		self.collapse:SetPoint("Bottom", self.scrollbar, "Top", 0, -3)
 	else
		Loffset, Roffset = 0,  -16
		self.scrollbar:SetPoint("TopLeft", self.Child, "TopRight", 0, 3)
		self.scrollbar:SetPoint("BottomLeft", self.Child, "BottomRight", 0, -3)
		self.collapse:SetPoint("Bottom", self.scrollbar, "Top", 0, -3)
 	end
 
 	Clicked()



	local pad = self.sets.padding/2 or 0
	self:SetSize(self.sets.padding + self.sets.width, self.sets.padding + self.sets.height)
	self.Child:SetPoint("Topleft", pad + Loffset, -pad - 16)
	self.Child:SetPoint("BottomRight", -pad + Roffset, pad)
	self:OnEvent()
end

function Quest:TrackerSetup()
	self.header = _G["WatchFrameHeader"]
	self.header:SetParent(self.Child)
	self.collapse = _G["WatchFrameCollapseExpandButton"]
	self.collapse:SetParent(self.Child)
	self.collapse.click = self.collapse:GetScript("OnClick")
	self.collapse:SetFrameLevel(self.collapse:GetFrameLevel()+3)
	Clicked = function()
 		self.header:ClearAllPoints()
		if not WatchFrame.userCollapsed then
 			if self.sets.OnLeft then
				self.header:SetPoint("BottomRight", self.Child ,"TopRight")
 			else
				self.header:SetPoint("BottomLeft", self.Child, "TopLeft")
			end		
		else
 			if self.sets.OnLeft then
				self.header:SetPoint("Left", self.collapse ,"Right")
 			else
				self.header:SetPoint("Right", self.collapse ,"Left")
			end	
		end
 	end
	self.collapse:SetScript("OnClick", function(s)
 		 WatchFrame_CollapseExpandButton_OnClick(s)
 		 Clicked()
 	end)

	self.tracker = _G["WatchFrame"]
	self.lines = _G["WatchFrameLines"]
	self.tracker:SetParent(self.contain)

	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("QUEST_LOG_UPDATE");
	self:RegisterEvent("TRACKED_ACHIEVEMENT_UPDATE");
	self:RegisterEvent("ITEM_PUSH");
	self:RegisterEvent("DISPLAY_SIZE_CHANGED");
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA");
	self:RegisterEvent("WORLD_MAP_UPDATE");
	self:RegisterEvent("QUEST_POI_UPDATE");
	self:RegisterEvent("PLAYER_MONEY");
	self:RegisterEvent("VARIABLES_LOADED");
	self:RegisterEvent("QUEST_AUTOCOMPLETE");
	self:RegisterEvent("UPDATE_SHAPESHIFT_COOLDOWN");
	self:RegisterEvent("UPDATE_SHAPESHIFT_FORM");
	self:RegisterEvent("UPDATE_SHAPESHIFT_USABLE");
	self:RegisterEvent("UPDATE_SHAPESHIFT_FORMS");
	self:SetScript("OnEvent", self.OnEvent)
end

function Quest:OnEvent()
	self.tracker:ClearAllPoints()
	self.tracker:SetPoint("TopLeft", 24 , -24)
	self.tracker:SetPoint("TopRight", -24 , -24)
	self.tracker:SetPoint("Bottom")
	self.contain:SetWidth(self.Child:GetWidth() - 19)

	WatchFrame:SetWidth(self.contain:GetWidth() or 1);
	WATCHFRAME_EXPANDEDWIDTH = self.contain:GetWidth() or 1
	WATCHFRAME_MAXLINEWIDTH =self.contain:GetWidth() or 1
	WatchFrame_Update();
	
	self.lines :ClearAllPoints()
	self.lines:SetAllPoints(self.contain)
	self.lines:SetPoint("TopLeft", self.contain, 24, 0)
	self.lines:SetPoint("BottomRight", self.contain, -24 , 0)
	
	for j = 1, WATCHFRAME_NUM_ITEMS do
		local item = _G["WatchFrameItem" .. j]
		if item then
			item:SetFrameLevel(9)
			local a, p, b, x , y = item:GetPoint()
			item:ClearAllPoints()
			item:SetPoint("TopLeft", p, "TopRight", 10 , -2)
		end
	end
end

function Quest:ScrollBar(enable, value)
	self.sets[value] = enable or nil
	self:Layout()
end

local function NewSizingSlider(p, name, min, max)
	local s = p:NewSlider(name, min, max, 1, OnShow)
	s.OnShow = function(self)
		self:SetValue(self:GetParent().owner.sets[string.lower(name)])
	end
	s.UpdateValue = function(self, value)
		local f = self:GetParent().owner
		f.sets[string.lower(name)] = value
		f:Layout()
	end
end

local function NewCheckButton(p, name, value)
	local check = p:NewCheckButton(name)
	check:SetScript('OnClick', function(self) p.owner:ScrollBar(self:GetChecked(), value) end)
	check:SetScript('OnShow', function(self) self:SetChecked(p.owner.sets[value]) end)
end

local function AddLayoutPanel(menu)
	local p = menu:NewPanel(LibStub('AceLocale-3.0'):GetLocale('Dominos-Config').Layout)
	p:NewOpacitySlider()
	p:NewFadeSlider()
	NewSizingSlider(p, "Width", 100, 300)
	NewSizingSlider(p, "Height", 120, GetScreenHeight())
	NewSizingSlider(p, "Padding", -16, 32)
	p:NewScaleSlider()
	NewCheckButton(p, "ScrollBar on Left", "OnLeft")
	NewCheckButton(p, "Disable ScrollBar", "disable")
end

function Quest:CreateMenu()
	local menu = Dominos:NewMenu(self.id)
	AddLayoutPanel(menu)
	self.menu = menu
end

function Quest:NewScrollFrame(parent)
	--scrollframe 
	frame = CreateFrame("ScrollFrame", "Dominos_QuestScrollFrame", parent) 
	frame:SetPoint("Center")

	--scrollbar 
	scrollbar = CreateFrame("Slider", "Dominos_QuestScrollbar", frame, "OptionsSliderTemplate")
	_G[scrollbar:GetName() .. 'High']:SetText("")
	_G[scrollbar:GetName() .. 'Low']:SetText("")
	scrollbar:SetOrientation("VERTICAL")
	
	scrollbar:SetMinMaxValues(1, 200) 
	scrollbar:SetValueStep(10) 
	scrollbar.scrollStep = 1 
	scrollbar:SetValue(0) 
	scrollbar:SetWidth(16) 
	scrollbar:SetScript("OnValueChanged", 
	function (self, value) 
		self:GetParent():SetVerticalScroll(value) 
	end)  

	local function scrollbar_sOnScroll(self, arg1)
		local step = (scrollbar:GetValueStep() * arg1)
		local value = scrollbar:GetValue()
		local minVal, maxVal = scrollbar:GetMinMaxValues()
		if step > 0 then
			scrollbar:SetValue(min(value-step, maxVal))
		else
			scrollbar:SetValue(max(value-step, minVal))
		end
	end

	local function scrollbar_OnScroll(x, arg1)
		local step = (scrollbar:GetValueStep() * arg1)
		local value = scrollbar:GetValue()
		local minVal, maxVal = scrollbar:GetMinMaxValues()
		if step > 0 then
			scrollbar:SetValue(min(value-step, maxVal))
		else
			scrollbar:SetValue(max(value-step, minVal))
		end
	end

	scrollbar:SetScript('OnMouseWheel', scrollbar_OnScroll)
	frame:SetScript('OnMouseWheel', scrollbar_OnScroll)
	
	--content frame 
	local content = CreateFrame("Frame", nil, frame)
	content:SetSize(128, 1000)
	frame:SetScrollChild(content)

	return frame, scrollbar, content
end
